@@ -3,6 +3,7 @@ import json
 import random
 import string
 import base64
+import asyncio
 from datetime import datetime
 
 from .log_config import logger
@@ -14,19 +15,18 @@ async def check_response(response, error_log):
         error_message = await response.aread()
         error_str = error_message.decode('utf-8', errors='replace')
         try:
-            error_json = json.loads(error_str)
+            error_json = await asyncio.to_thread(json.loads, error_str)
         except json.JSONDecodeError:
             error_json = error_str
         return {"error": f"{error_log} HTTP Error", "status_code": response.status_code, "details": error_json}
     return None
 
-def gemini_json_poccess(response_str):
+async def gemini_json_poccess(response_json):
     promptTokenCount = 0
     candidatesTokenCount = 0
     totalTokenCount = 0
     image_base64 = None
 
-    response_json = json.loads(response_str)
     json_data = safe_get(response_json, "candidates", 0, "content", default=None)
     finishReason = safe_get(response_json, "candidates", 0 , "finishReason", default=None)
     if finishReason:
@@ -47,15 +47,16 @@ def gemini_json_poccess(response_str):
 
     function_call_name = safe_get(json_data, "functionCall", "name", default=None)
     function_full_response = safe_get(json_data, "functionCall", "args", default="")
-    function_full_response = json.dumps(function_full_response) if function_full_response else None
+    function_full_response = await asyncio.to_thread(json.dumps, function_full_response) if function_full_response else None
 
     blockReason = safe_get(json_data, 0, "promptFeedback", "blockReason", default=None)
 
     return is_thinking, reasoning_content, content, image_base64, function_call_name, function_full_response, finishReason, blockReason, promptTokenCount, candidatesTokenCount, totalTokenCount
 
-async def fetch_gemini_response_stream(client, url, headers, payload, model):
+async def fetch_gemini_response_stream(client, url, headers, payload, model, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
         error_message = await check_response(response, "fetch_gemini_response_stream")
         if error_message:
             yield error_message
@@ -75,7 +76,7 @@ async def fetch_gemini_response_stream(client, url, headers, payload, model):
                 if line.startswith("data: "):
                     parts_json = line.lstrip("data: ").strip()
                     try:
-                        json.loads(parts_json)
+                        response_json = await asyncio.to_thread(json.loads, parts_json)
                     except json.JSONDecodeError:
                         logger.error(f"JSON decode error: {parts_json}")
                         continue
@@ -83,12 +84,12 @@ async def fetch_gemini_response_stream(client, url, headers, payload, model):
                     parts_json += line
                     parts_json = parts_json.lstrip("[,")
                     try:
-                        json.loads(parts_json)
+                        response_json = await asyncio.to_thread(json.loads, parts_json)
                     except json.JSONDecodeError:
                         continue
 
                 # https://ai.google.dev/api/generate-content?hl=zh-cn#FinishReason
-                is_thinking, reasoning_content, content, image_base64, function_call_name, function_full_response, finishReason, blockReason, promptTokenCount, candidatesTokenCount, totalTokenCount = gemini_json_poccess(parts_json)
+                is_thinking, reasoning_content, content, image_base64, function_call_name, function_full_response, finishReason, blockReason, promptTokenCount, candidatesTokenCount, totalTokenCount = await gemini_json_poccess(response_json)
 
                 if is_thinking:
                     sse_string = await generate_sse_response(timestamp, model, reasoning_content=reasoning_content)
@@ -122,9 +123,10 @@ async def fetch_gemini_response_stream(client, url, headers, payload, model):
 
     yield "data: [DONE]" + end_of_line
 
-async def fetch_vertex_claude_response_stream(client, url, headers, payload, model):
+async def fetch_vertex_claude_response_stream(client, url, headers, payload, model, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
         error_message = await check_response(response, "fetch_vertex_claude_response_stream")
         if error_message:
             yield error_message
@@ -159,7 +161,7 @@ async def fetch_vertex_claude_response_stream(client, url, headers, payload, mod
 
                 if line and '\"text\": \"' in line and is_finish == False:
                     try:
-                        json_data = json.loads( "{" + line.strip().rstrip(",") + "}")
+                        json_data = await asyncio.to_thread(json.loads, "{" + line.strip().rstrip(",") + "}")
                         content = json_data.get('text', '')
                         sse_string = await generate_sse_response(timestamp, model, content=content)
                         yield sse_string
@@ -176,12 +178,12 @@ async def fetch_vertex_claude_response_stream(client, url, headers, payload, mod
                     function_full_response += line
 
         if need_function_call:
-            function_call = json.loads(function_full_response)
+            function_call = await asyncio.to_thread(json.loads, function_full_response)
             function_call_name = function_call["name"]
             function_call_id = function_call["id"]
             sse_string = await generate_sse_response(timestamp, model, content=None, tools_id=function_call_id, function_call_name=function_call_name)
             yield sse_string
-            function_full_response = json.dumps(function_call["input"])
+            function_full_response = await asyncio.to_thread(json.dumps, function_call["input"])
             sse_string = await generate_sse_response(timestamp, model, content=None, tools_id=function_call_id, function_call_name=None, function_call_content=function_full_response)
             yield sse_string
 
@@ -190,14 +192,15 @@ async def fetch_vertex_claude_response_stream(client, url, headers, payload, mod
 
     yield "data: [DONE]" + end_of_line
 
-async def fetch_gpt_response_stream(client, url, headers, payload):
+async def fetch_gpt_response_stream(client, url, headers, payload, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
     random.seed(timestamp)
     random_str = ''.join(random.choices(string.ascii_letters + string.digits, k=29))
     is_thinking = False
     has_send_thinking = False
     ark_tag = False
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
         error_message = await check_response(response, "fetch_gpt_response_stream")
         if error_message:
             yield error_message
@@ -210,10 +213,13 @@ async def fetch_gpt_response_stream(client, url, headers, payload):
             while "\n" in buffer:
                 line, buffer = buffer.split("\n", 1)
                 # logger.info("line: %s", repr(line))
+                if line.startswith(": keepalive"):
+                    yield line + end_of_line
+                    continue
                 if line and not line.startswith(":") and (result:=line.lstrip("data: ").strip()):
                     if result.strip() == "[DONE]":
                         break
-                    line = json.loads(result)
+                    line = await asyncio.to_thread(json.loads, result)
                     line['id'] = f"chatcmpl-{random_str}"
 
                     # 处理 <think> 标签
@@ -269,6 +275,11 @@ async def fetch_gpt_response_stream(client, url, headers, payload):
 
                     no_stream_content = safe_get(line, "choices", 0, "message", "content", default=None)
                     openrouter_reasoning = safe_get(line, "choices", 0, "delta", "reasoning", default="")
+                    openrouter_base64_image = safe_get(line, "choices", 0, "delta", "images", 0, "image_url", "url", default="")
+                    if openrouter_base64_image:
+                        sse_string = await generate_sse_response(timestamp, payload["model"], content=f"\n\n![image]({openrouter_base64_image})")
+                        yield sse_string
+                        continue
                     azure_databricks_claude_summary_content = safe_get(line, "choices", 0, "delta", "content", 0, "summary", 0, "text", default="")
                     azure_databricks_claude_signature_content = safe_get(line, "choices", 0, "delta", "content", 0, "summary", 0, "signature", default="")
                     # print("openrouter_reasoning", repr(openrouter_reasoning), openrouter_reasoning.endswith("\\\\"), openrouter_reasoning.endswith("\\"))
@@ -303,15 +314,17 @@ async def fetch_gpt_response_stream(client, url, headers, payload):
                     else:
                         if no_stream_content:
                             del line["choices"][0]["message"]
-                        yield "data: " + json.dumps(line).strip() + end_of_line
+                        json_line = await asyncio.to_thread(json.dumps, line)
+                        yield "data: " + json_line.strip() + end_of_line
     yield "data: [DONE]" + end_of_line
 
-async def fetch_azure_response_stream(client, url, headers, payload):
+async def fetch_azure_response_stream(client, url, headers, payload, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
     is_thinking = False
     has_send_thinking = False
     ark_tag = False
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
         error_message = await check_response(response, "fetch_azure_response_stream")
         if error_message:
             yield error_message
@@ -327,7 +340,7 @@ async def fetch_azure_response_stream(client, url, headers, payload):
                 if line and not line.startswith(":") and (result:=line.lstrip("data: ").strip()):
                     if result.strip() == "[DONE]":
                         break
-                    line = json.loads(result)
+                    line = await asyncio.to_thread(json.loads, result)
                     no_stream_content = safe_get(line, "choices", 0, "message", "content", default="")
                     content = safe_get(line, "choices", 0, "delta", "content", default="")
 
@@ -359,12 +372,14 @@ async def fetch_azure_response_stream(client, url, headers, payload):
                     else:
                         if no_stream_content:
                             del line["choices"][0]["message"]
-                        yield "data: " + json.dumps(line).strip() + end_of_line
+                        json_line = await asyncio.to_thread(json.dumps, line)
+                        yield "data: " + json_line.strip() + end_of_line
     yield "data: [DONE]" + end_of_line
 
-async def fetch_cloudflare_response_stream(client, url, headers, payload, model):
+async def fetch_cloudflare_response_stream(client, url, headers, payload, model, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
         error_message = await check_response(response, "fetch_cloudflare_response_stream")
         if error_message:
             yield error_message
@@ -380,17 +395,18 @@ async def fetch_cloudflare_response_stream(client, url, headers, payload, model)
                     line = line.lstrip("data: ")
                     if line == "[DONE]":
                         break
-                    resp: dict = json.loads(line)
+                    resp: dict = await asyncio.to_thread(json.loads, line)
                     message = resp.get("response")
                     if message:
                         sse_string = await generate_sse_response(timestamp, model, content=message)
                         yield sse_string
     yield "data: [DONE]" + end_of_line
 
-async def fetch_cohere_response_stream(client, url, headers, payload, model):
+async def fetch_cohere_response_stream(client, url, headers, payload, model, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
-        error_message = await check_response(response, "fetch_gpt_response_stream")
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
+        error_message = await check_response(response, "fetch_cohere_response_stream")
         if error_message:
             yield error_message
             return
@@ -401,7 +417,7 @@ async def fetch_cohere_response_stream(client, url, headers, payload, model):
             while "\n" in buffer:
                 line, buffer = buffer.split("\n", 1)
                 # logger.info("line: %s", repr(line))
-                resp: dict = json.loads(line)
+                resp: dict = await asyncio.to_thread(json.loads, line)
                 if resp.get("is_finished") == True:
                     break
                 if resp.get("event_type") == "text-generation":
@@ -410,9 +426,10 @@ async def fetch_cohere_response_stream(client, url, headers, payload, model):
                     yield sse_string
     yield "data: [DONE]" + end_of_line
 
-async def fetch_claude_response_stream(client, url, headers, payload, model):
+async def fetch_claude_response_stream(client, url, headers, payload, model, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
         error_message = await check_response(response, "fetch_claude_response_stream")
         if error_message:
             yield error_message
@@ -427,7 +444,7 @@ async def fetch_claude_response_stream(client, url, headers, payload, model):
                 # logger.info(line)
 
                 if line.startswith("data:") and (line := line.lstrip("data: ")):
-                    resp: dict = json.loads(line)
+                    resp: dict = await asyncio.to_thread(json.loads, line)
 
                     input_tokens = input_tokens or safe_get(resp, "message", "usage", "input_tokens", default=0)
                     # cache_creation_input_tokens = safe_get(resp, "message", "usage", "cache_creation_input_tokens", default=0)
@@ -463,9 +480,10 @@ async def fetch_claude_response_stream(client, url, headers, payload, model):
 
     yield "data: [DONE]" + end_of_line
 
-async def fetch_aws_response_stream(client, url, headers, payload, model):
+async def fetch_aws_response_stream(client, url, headers, payload, model, timeout):
     timestamp = int(datetime.timestamp(datetime.now()))
-    async with client.stream('POST', url, headers=headers, json=payload) as response:
+    json_payload = await asyncio.to_thread(json.dumps, payload)
+    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
         error_message = await check_response(response, "fetch_aws_response_stream")
         if error_message:
             yield error_message
@@ -486,7 +504,7 @@ async def fetch_aws_response_stream(client, url, headers, payload, model):
                 if not json_match:
                     continue
                 try:
-                    chunk_data = json.loads(json_match.group(0).lstrip('event'))
+                    chunk_data = await asyncio.to_thread(json.loads, json_match.group(0).lstrip('event'))
                 except json.JSONDecodeError:
                     logger.error(f"DEBUG json.JSONDecodeError: {json_match.group(0).lstrip('event')!r}")
                     continue
@@ -496,7 +514,7 @@ async def fetch_aws_response_stream(client, url, headers, payload, model):
                     # 解码 Base64 编码的字节
                     decoded_bytes = base64.b64decode(chunk_data["bytes"])
                     # 将解码后的字节再次解析为 JSON
-                    payload_chunk = json.loads(decoded_bytes.decode('utf-8'))
+                    payload_chunk = await asyncio.to_thread(json.loads, decoded_bytes.decode('utf-8'))
                     # print(f"DEBUG payload_chunk: {payload_chunk!r}")
 
                     text = safe_get(payload_chunk, "delta", "text", default="")
@@ -514,13 +532,14 @@ async def fetch_aws_response_stream(client, url, headers, payload, model):
 
     yield "data: [DONE]" + end_of_line
 
-async def fetch_response(client, url, headers, payload, engine, model):
+async def fetch_response(client, url, headers, payload, engine, model, timeout=200):
     response = None
     if payload.get("file"):
         file = payload.pop("file")
-        response = await client.post(url, headers=headers, data=payload, files={"file": file})
+        response = await client.post(url, headers=headers, data=payload, files={"file": file}, timeout=timeout)
     else:
-        response = await client.post(url, headers=headers, json=payload)
+        json_payload = await asyncio.to_thread(json.dumps, payload)
+        response = await client.post(url, headers=headers, content=json_payload, timeout=timeout)
     error_message = await check_response(response, "fetch_response")
     if error_message:
         yield error_message
@@ -530,7 +549,8 @@ async def fetch_response(client, url, headers, payload, engine, model):
         yield response.read()
 
     elif engine == "gemini" or engine == "vertex-gemini" or engine == "aws":
-        response_json = response.json()
+        response_bytes = await response.aread()
+        response_json = await asyncio.to_thread(json.loads, response_bytes)
         # print("response_json", json.dumps(response_json, indent=4, ensure_ascii=False))
 
         if isinstance(response_json, str):
@@ -585,7 +605,8 @@ async def fetch_response(client, url, headers, payload, engine, model):
         yield await generate_no_stream_response(timestamp, model, content=content, tools_id=None, function_call_name=function_call_name, function_call_content=function_call_content, role=role, total_tokens=total_tokens, prompt_tokens=prompt_tokens, completion_tokens=candidates_tokens, reasoning_content=reasoning_content, image_base64=image_base64)
 
     elif engine == "claude":
-        response_json = response.json()
+        response_bytes = await response.aread()
+        response_json = await asyncio.to_thread(json.loads, response_bytes)
         # print("response_json", json.dumps(response_json, indent=4, ensure_ascii=False))
 
         content = safe_get(response_json, "content", 0, "text")
@@ -604,7 +625,8 @@ async def fetch_response(client, url, headers, payload, engine, model):
         yield await generate_no_stream_response(timestamp, model, content=content, tools_id=tools_id, function_call_name=function_call_name, function_call_content=function_call_content, role=role, total_tokens=total_tokens, prompt_tokens=prompt_tokens, completion_tokens=output_tokens)
 
     elif engine == "azure":
-        response_json = response.json()
+        response_bytes = await response.aread()
+        response_json = await asyncio.to_thread(json.loads, response_bytes)
         # 删除 content_filter_results
         if "choices" in response_json:
             for choice in response_json["choices"]:
@@ -618,34 +640,58 @@ async def fetch_response(client, url, headers, payload, engine, model):
         yield response_json
 
     elif "dashscope.aliyuncs.com" in url and "multimodal-generation" in url:
-        response_json = response.json()
+        response_bytes = await response.aread()
+        response_json = await asyncio.to_thread(json.loads, response_bytes)
         content = safe_get(response_json, "output", "choices", 0, "message", "content", 0, default=None)
         yield content
+
+    elif "embedContent" in url:
+        response_bytes = await response.aread()
+        response_json = await asyncio.to_thread(json.loads, response_bytes)
+        content = safe_get(response_json, "embedding", "values", default=[])
+        response_embedContent = {
+            "object": "list",
+            "data": [
+                {
+                    "object": "embedding",
+                    "embedding":content,
+                    "index": 0
+                }
+            ],
+            "model": model,
+            "usage": {
+                "prompt_tokens": 0,
+                "total_tokens": 0
+            }
+        }
+
+        yield response_embedContent
     else:
-        response_json = response.json()
+        response_bytes = await response.aread()
+        response_json = await asyncio.to_thread(json.loads, response_bytes)
         yield response_json
 
-async def fetch_response_stream(client, url, headers, payload, engine, model):
+async def fetch_response_stream(client, url, headers, payload, engine, model, timeout=200):
     if engine == "gemini" or engine == "vertex-gemini":
-        async for chunk in fetch_gemini_response_stream(client, url, headers, payload, model):
+        async for chunk in fetch_gemini_response_stream(client, url, headers, payload, model, timeout):
             yield chunk
     elif engine == "claude" or engine == "vertex-claude":
-        async for chunk in fetch_claude_response_stream(client, url, headers, payload, model):
+        async for chunk in fetch_claude_response_stream(client, url, headers, payload, model, timeout):
             yield chunk
     elif engine == "aws":
-        async for chunk in fetch_aws_response_stream(client, url, headers, payload, model):
+        async for chunk in fetch_aws_response_stream(client, url, headers, payload, model, timeout):
             yield chunk
     elif engine == "gpt" or engine == "openrouter" or engine == "azure-databricks":
-        async for chunk in fetch_gpt_response_stream(client, url, headers, payload):
+        async for chunk in fetch_gpt_response_stream(client, url, headers, payload, timeout):
             yield chunk
     elif engine == "azure":
-        async for chunk in fetch_azure_response_stream(client, url, headers, payload):
+        async for chunk in fetch_azure_response_stream(client, url, headers, payload, timeout):
             yield chunk
     elif engine == "cloudflare":
-        async for chunk in fetch_cloudflare_response_stream(client, url, headers, payload, model):
+        async for chunk in fetch_cloudflare_response_stream(client, url, headers, payload, model, timeout):
             yield chunk
     elif engine == "cohere":
-        async for chunk in fetch_cohere_response_stream(client, url, headers, payload, model):
+        async for chunk in fetch_cohere_response_stream(client, url, headers, payload, model, timeout):
             yield chunk
     else:
-        raise ValueError("Unknown response")
\ No newline at end of file
+        raise ValueError("Unknown response")
