@@ -63,11 +63,13 @@ class BaseAPI:
         else:
             self.audio_speech: str = urlunparse(parsed_url[:2] + (before_v1 + "audio/speech",) + ("",) * 3)
 
-        if parsed_url.hostname == "generativelanguage.googleapis.com":
+        if parsed_url.path.endswith("/v1beta") or \
+        (parsed_url.netloc == 'generativelanguage.googleapis.com' and "openai/chat/completions" not in parsed_url.path):
+            before_v1 = parsed_url.path.split("/v1")[0]
             self.base_url = api_url
             self.v1_url = api_url
             self.chat_url = api_url
-            self.embeddings = api_url
+            self.embeddings = urlunparse(parsed_url[:2] + (before_v1 + "/v1beta/embeddings",) + ("",) * 3)
 
 def get_engine(provider, endpoint=None, original_model=""):
     parsed_url = urlparse(provider['base_url'])
@@ -75,7 +77,6 @@ def get_engine(provider, endpoint=None, original_model=""):
     engine = None
     stream = None
     if parsed_url.path.endswith("/v1beta") or \
-    parsed_url.path.endswith("/v1") or \
     (parsed_url.netloc == 'generativelanguage.googleapis.com' and "openai/chat/completions" not in parsed_url.path):
         engine = "gemini"
     elif parsed_url.netloc.rstrip('/').endswith('aiplatform.googleapis.com') or \
@@ -166,7 +167,7 @@ def get_proxy(proxy, client_config = {}):
             }
     return client_config
 
-def update_initial_model(provider):
+async def update_initial_model(provider):
     try:
         engine, stream_mode = get_engine(provider, endpoint=None, original_model="")
         # print("engine", engine, provider)
@@ -178,8 +179,8 @@ def update_initial_model(provider):
             before_v1 = api_url.split("/v1beta")[0]
             url = before_v1 + "/v1beta/models"
             params = {"key": api}
-            with httpx.Client(**client_config) as client:
-                response = client.get(url, params=params)
+            async with httpx.AsyncClient(**client_config) as client:
+                response = await client.get(url, params=params)
 
             original_models = response.json()
             if original_models.get("error"):
@@ -199,11 +200,11 @@ def update_initial_model(provider):
                 headers = {"x-api-key": api, "anthropic-version": "2023-06-01"}
             else:
                 headers = {"Authorization": f"Bearer {api}"}
-            response = httpx.get(
-                endpoint_models_url,
-                headers=headers,
-                **client_config
-            )
+            async with httpx.AsyncClient(**client_config) as client:
+                response = await client.get(
+                    endpoint_models_url,
+                    headers=headers,
+                )
             models = response.json()
             if models.get("error"):
                 logger.error({"error": models.get("error"), "endpoint": endpoint_models_url, "api": api})
@@ -242,7 +243,8 @@ def parse_rate_limit(limit_string):
         'h': 3600, 'hr': 3600, 'hour': 3600,
         'd': 86400, 'day': 86400,
         'mo': 2592000, 'month': 2592000,
-        'y': 31536000, 'year': 31536000
+        'y': 31536000, 'year': 31536000,
+        'tpr': -1,
     }
 
     # 处理多个限制条件
@@ -376,6 +378,32 @@ class ThreadSafeCircularList:
                     logger.warning(f"All API keys are rate limited!")
                     raise HTTPException(status_code=429, detail="Too many requests")
 
+    async def is_tpr_exceeded(self, model: str = None, tokens: int = 0) -> bool:
+        """Checks if the request exceeds the TPR (Tokens Per Request) limit."""
+        if not tokens:
+            return False
+
+        async with self.lock:
+            rate_limit = None
+            model_key = model or "default"
+            if model and model_key in self.rate_limits:
+                rate_limit = self.rate_limits[model_key]
+            else:
+                # fuzzy match
+                for limit_model in self.rate_limits:
+                    if limit_model != "default" and model and limit_model in model:
+                        rate_limit = self.rate_limits[limit_model]
+                        break
+            if rate_limit is None:
+                rate_limit = self.rate_limits.get("default", [])
+
+            for limit_count, limit_period in rate_limit:
+                if limit_period == -1:  # TPR limit
+                    if tokens > limit_count:
+                        # logger.warning(f"API provider for model {model_key} exceeds TPR limit ({tokens}/{limit_count}).")
+                        return True
+        return False
+
     async def is_all_rate_limited(self, model: str = None) -> bool:
         """检查是否所有的items都被速率限制
 
@@ -518,7 +546,7 @@ async def generate_sse_response(timestamp, model, content=None, tools_id=None, f
         sample_data["choices"][0]["delta"] = {}
         sample_data["choices"][0]["finish_reason"] = stop
 
-    json_data = json.dumps(sample_data, ensure_ascii=False)
+    json_data = await asyncio.to_thread(json.dumps, sample_data, ensure_ascii=False)
     # print("json_data", json.dumps(sample_data, indent=4, ensure_ascii=False))
 
     # 构建SSE响应
@@ -557,6 +585,9 @@ async def generate_no_stream_response(timestamp, model, content=None, tools_id=N
     if function_call_name:
         if not tools_id:
             tools_id = f"call_{random_str}"
+
+        arguments_json = await asyncio.to_thread(json.dumps, function_call_content, ensure_ascii=False)
+
         sample_data = {
             "id": f"chatcmpl-{random_str}",
             "object": "chat.completion",
@@ -574,7 +605,7 @@ async def generate_no_stream_response(timestamp, model, content=None, tools_id=N
                             "type": "function",
                             "function": {
                                 "name": function_call_name,
-                                "arguments": json.dumps(function_call_content, ensure_ascii=False)
+                                "arguments": arguments_json
                             }
                         }
                     ],
@@ -605,7 +636,7 @@ async def generate_no_stream_response(timestamp, model, content=None, tools_id=N
     if total_tokens:
         sample_data["usage"] = {"prompt_tokens": prompt_tokens, "completion_tokens": completion_tokens, "total_tokens": total_tokens}
 
-    json_data = json.dumps(sample_data, ensure_ascii=False)
+    json_data = await asyncio.to_thread(json.dumps, sample_data, ensure_ascii=False)
     # print("json_data", json.dumps(sample_data, indent=4, ensure_ascii=False))
 
     return json_data
