@@ -6,6 +6,7 @@ import httpx
 import asyncio
 import logging
 import inspect
+from collections import defaultdict
 from typing import Union, Optional, Callable
 
 from .base import BaseLLM
@@ -97,9 +98,7 @@ class chatgpt(BaseLLM):
         Initialize Chatbot with API key (from https://platform.openai.com/account/api-keys)
         """
         super().__init__(api_key, engine, api_url, system_prompt, proxy, timeout, max_tokens, temperature, top_p, presence_penalty, frequency_penalty, reply_count, truncate_limit, use_plugins=use_plugins, print_log=print_log)
-        self.conversation: dict[str, Messages] = {
-            "default": Messages(SystemMessage(self.system_prompt)),
-        }
+        self.conversation: dict[str, Messages] = defaultdict(lambda: Messages(SystemMessage(self.system_prompt)))
         if cache_messages:
             self.conversation["default"] = cache_messages
         self.function_calls_counter = {}
@@ -188,9 +187,9 @@ class chatgpt(BaseLLM):
                             ])
                     self.conversation[convo_id].append(AssistantMessage(convert_functions_to_xml(function_arguments)))
                     if image_message_list:
-                        self.conversation[convo_id].append(UserMessage(message + image_message_list))
+                        self.conversation[convo_id].append(UserMessage(message + image_message_list, Texts("\n\nYour message **must** end with [done] to signify the end of your output.", name="done", visible=self.check_done)))
                     else:
-                        self.conversation[convo_id].append(UserMessage(message))
+                        self.conversation[convo_id].append(UserMessage(message, Texts("\n\nYour message **must** end with [done] to signify the end of your output.", name="done", visible=self.check_done)))
                 else:
                     self.conversation[convo_id].append(AssistantMessage("我已经执行过这个工具了，接下来我需要做什么？"))
         else:
@@ -476,7 +475,9 @@ class chatgpt(BaseLLM):
                 # 删除 task_complete 跟其他工具一起调用的情况，因为 task_complete 必须单独调用
                 if len(function_parameter) > 1:
                     function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") != "task_complete"]
-                    function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") != "get_task_result"]
+                    # 仅当存在其他工具时，才删除 get_task_result
+                    if any(tool.get("function_name") != "get_task_result" for tool in function_parameter):
+                        function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") != "get_task_result"]
                 if len(function_parameter) == 1 and function_parameter[0].get("function_name", "") == "task_complete":
                     raise TaskComplete(safe_get(function_parameter, 0, "parameter", "message", default="The task has been completed."))
 
@@ -600,9 +601,6 @@ class chatgpt(BaseLLM):
                 else:
                     all_responses.append(f"[{tool_name}({tool_args}) Result]:\n\n{tool_response}")
 
-            if self.check_done:
-                all_responses.append("Your message **must** end with [done] to signify the end of your output.")
-
             # 合并所有工具响应
             function_response = "\n\n".join(all_responses).strip()
             if missing_required_params:
